# -*- coding: utf-8 -*-

import numpy as np
from numpy.linalg import inv
import GydImplementation
import GyuImplementation
import JuuImplementation
import JudImplementation
import Wd
import Wn
ep = 1e-8 #perturb value

#Get Gyu
Gmat = GyuImplementation.GetGyMatrix(1,5,ep)
#Get Juu
Juu = JuuImplementation.Juu(1,5, ep)
#Get Jud
Jud = JudImplementation.Jud(1,5,1,5,ep)
#Get Gyd
Gmatd = GydImplementation.GetGydMatrix(1,5,ep)
#Obtain F from the linear realationship
Fmat = np.dot(-Gmat,np.dot(inv(Juu), Jud)) + Gmatd
#Get Wd
Wd = Wd.Wd()
#Get Wn
Wn = Wn.Wn()




#####################Optimal measurement combinations############################## 
#Worst case Branch and Bound, annulus 2 and bottomhole 2
F1 = np.array([[Fmat[1][0], Fmat[1][1]],[Fmat[2][0], Fmat[2][1]]])
FWd1 = np.matmul(F1, Wd)
Wn1 = np.array([[Wn[1][1], 0], [0, Wn[2][2]]])
Y1 = np.concatenate((FWd1, Wn1.T), axis = 1)
#Y = [[-0.13019906  0.05559134  0.00253839  0.        ]
 #   [-0.34231843  0.02344421  0.          0.00343078]]
Gy1 = np.array([[Gmat[1][0], Gmat[1][1]], [Gmat[2][0], Gmat[2][1]]])
Yt1 = np.transpose(Y1)
h1 = np.matmul(inv(np.matmul(Y1,Yt1)),Gy1).transpose()
print(h1)
print(Y1)
#      ai2          Bh2
#[[ 3583.10465771 -1449.41787407]
# [ -146.18640435    67.56021995]]

#Average loss Branch and Bound, annulus 2 and annulus 6
F2 = np.array([[Fmat[1][0], Fmat[1][1]],[Fmat[4][0], Fmat[4][1]]])
print(F2)
FWd2 = np.matmul(F2, Wd)
Wn2 = np.array([[Wn[1][1], 0], [0, Wn[4][4]]])
Y2 = np.concatenate((FWd2, Wn2.T), axis = 1)
Gy2 = np.array([[Gmat[1][0], Gmat[1][1]], [Gmat[4][0], Gmat[4][1]]])
Yt2 = np.transpose(Y2)
h2 = np.matmul(inv(np.matmul(Y2,Yt2)),Gy2).transpose()
#Y = [[-0.13019906  0.05559134  0.00253839  0.        ]
 #[ 0.02353136 -0.10664106  0.          0.0025349 ]]
print(h2)
print(Y2)
#      ai2            ai6
#[[413.16600486 326.50490982]
 #[305.9743484  663.475804  ]]

#Worst case Partial Branch and Bound(2var), discharge pressure and annulus 6
F3 = np.array([[Fmat[7][0], Fmat[7][1]],[Fmat[4][0], Fmat[4][1]]])
FWd3 = np.matmul(F3, Wd)
Wn3 = np.array([[ Wn[7][7], 0], [0,  Wn[4][4]]])
Y3 = np.concatenate((FWd3, Wn3.T), axis = 1)
Gy3 = np.array([[Gmat[7][0], Gmat[7][1]], [Gmat[4][0], Gmat[4][1]]])
Yt3 = np.transpose(Y3)
h3 = np.matmul(inv(np.matmul(Y3,Yt3)),Gy3).transpose()
#Y = [[-0.02940493 -0.06080479  0.00396529  0.        ]
# [ 0.02353136 -0.10664106  0.          0.0025349 ]]
print(h3)
print(Y3)
#     pdisch        ai6
#[[-6976.59161527  3401.78133543]
# [-8339.24832552  4481.00108404]]

#Worst case Partial Branch and Bound(3var), discharge pressure and annulus 6 and bottomhole 6
F4 = np.array([[Fmat[7][0], Fmat[7][1]],[Fmat[4][0], Fmat[4][1]], [Fmat[5][0], Fmat[5][1]]])
FWd4 = np.matmul(F4, Wd)
Wn4 = np.array([[Wn[7][7], 0,0], [0, Wn[4][4],0], [0, 0, Wn[5][5]]])
Y4 = np.concatenate((FWd4, Wn4.T), axis = 1)
Gy4 = np.array([[Gmat[7][0], Gmat[7][1]], [Gmat[4][0], Gmat[4][1]], [Gmat[5][0], Gmat[5][1]]])
Yt4 = np.transpose(Y4)
h4 = np.matmul(inv(np.matmul(Y4,Yt4)),Gy4).transpose()
#Y = [[-0.02940493 -0.06080479  0.00396529  0.          0.        ]
 #[ 0.02353136 -0.10664106  0.          0.0025349   0.        ]
 #[ 0.09889795 -0.2016167   0.          0.          0.00344215]]
print(h4)
print(Y4)
#        DischP         ai6            bh6
#[[-234922.1903769   476847.10061302 -181679.62737955]
# [-549333.90812753 1128132.39347296 -431189.32210637]]

#Worst case Partial Branch and Bound(4var), discharge pressure and annulus 6 and bottomhole 6 and annulus 2
F5 = np.array([[Fmat[7][0], Fmat[7][1]],[Fmat[4][0], Fmat[4][1]], [Fmat[5][0], Fmat[5][1]],[Fmat[1][0], Fmat[1][1]]])
FWd5 = np.matmul(F5, Wd)
Wn5 = np.array([[Wn[7][7], 0,0,0], [0, Wn[4][4],0,0], [0, 0, Wn[5][5],0], [0, 0, 0, Wn[1][1]]])
Y5 = np.concatenate((FWd5, Wn5.T), axis = 1)
Gy5 = np.array([[Gmat[7][0], Gmat[7][1]], [Gmat[4][0], Gmat[4][1]], [Gmat[5][0], Gmat[5][1]], [Gmat[1][0], Gmat[1][1]]])
Yt5 = np.transpose(Y5)
h5 = np.matmul(inv(np.matmul(Y5,Yt5)),Gy5).transpose()
print(h5)
#Y = [[-0.02940493 -0.06080479  0.00396529  0.          0.          0.        ]
# [ 0.02353136 -0.10664106  0.          0.0025349   0.          0.        ]
# [ 0.09889795 -0.2016167   0.          0.          0.00344215  0.        ]
# [-0.13019906  0.05559134  0.          0.          0.          0.00253839]]
print(Y5)
#      DischP             ai6             bh6               ai2
#[[-805763.43006082  131567.44007523  291161.30688082  427066.88982393]
# [-691151.33978905 1042352.54566528 -313718.6801493   106098.72807374]]

#Average loss Partial Branch and Bound(2var), discharge pressure and annulus 2
F6 = np.array([[Fmat[7][0], Fmat[7][1]],[Fmat[1][0], Fmat[1][1]]])
FWd6 = np.matmul(F6, Wd)
Wn6 = np.array([[Wn[7][7], 0], [0, Wn[1][1]]])
Y6 = np.concatenate((FWd6, Wn6.T), axis = 1)
Gy6 = np.array([[Gmat[7][0], Gmat[7][1]], [Gmat[1][0], Gmat[1][1]]])
Yt6 = np.transpose(Y6)
h6 = np.matmul(inv(np.matmul(Y6,Yt6)),Gy6).transpose()
print(Y6)
#Y = [[-0.02940493 -0.06080479  0.00396529  0.        ]
# [-0.13019906  0.05559134  0.          0.00253839]]
print(h6)
#    DischP        ai2
#[[-2704.12302706   327.18670482]
# [-2675.80879344    68.22602697]]


#Average loss Partial Branch and Bound(3var), discharge pressure and annulus 2 and bottomhole 2
F7 = np.array([[Fmat[7][0], Fmat[7][1]],[Fmat[1][0], Fmat[1][1]], [Fmat[2][0], Fmat[2][1]]])
FWd7 = np.matmul(F7, Wd)
Wn7 = np.array([[Wn[7][7], 0,0], [0, Wn[1][1],0], [0, 0, Wn[2][2]]])
Y7 = np.concatenate((FWd7, Wn7.T), axis = 1)
Gy7 = np.array([[Gmat[7][0], Gmat[7][1]], [Gmat[1][0], Gmat[1][1]], [Gmat[2][0], Gmat[2][1]]])
Yt7 = np.transpose(Y7)
h7 = np.matmul(inv(np.matmul(Y7,Yt7)),Gy7).transpose()
print(Y7)
#Y = [[-0.02940493 -0.06080479  0.00396529  0.          0.        ]
# [-0.13019906  0.05559134  0.          0.00253839  0.        ]
# [-0.34231843  0.02344421  0.          0.          0.00343078]]
print(h7)
#      DischP           ai2            bh2
#[[ -41674.87829334  -52288.11647798   23376.24685595]
# [-403800.65877812 -541499.5586333   240611.02868619]]

#Worst case Partial Branch and Bound(4var), discharge pressure and wellhead 6 and bottomhole 2 and annulus 2
F8 = np.array([[Fmat[7][0], Fmat[7][1]],[Fmat[3][0], Fmat[3][1]], [Fmat[2][0], Fmat[2][1]],[Fmat[1][0], Fmat[1][1]]])
FWd8 = np.matmul(F8, Wd)
Wn8 = np.array([[Wn[7][7], 0,0,0], [0, Wn[3][3],0,0], [0, 0, Wn[2][2],0], [0, 0, 0, Wn[1][1]]])
Y8 = np.concatenate((FWd8, Wn8.T), axis = 1)
Gy8 = np.array([[Gmat[7][0], Gmat[7][1]], [Gmat[3][0], Gmat[3][1]], [Gmat[2][0], Gmat[2][1]], [Gmat[1][0], Gmat[1][1]]])
Yt8 = np.transpose(Y8)
h8 = np.matmul(inv(np.matmul(Y8,Yt8)),Gy8).transpose()
print(Y8)
#Y = [[-0.02940493 -0.06080479  0.00396529  0.          0.          0.        ]
 #[ 0.04422688  0.13007002  0.          0.00201958  0.          0.        ]
 #[-0.34231843  0.02344421  0.          0.          0.00343078  0.        ]
 #[-0.13019906  0.05559134  0.          0.          0.          0.00253839]]
print(h8)
#    DischP            Wh6              BH2            ai2
#[[-686015.79742427 -613181.02538563 -334371.87860306  825709.36229966]
# [-659260.97418188 -243106.42626252   98775.47820129 -193401.98141469]]





