
import numpy as np 
from sys import path
path.append(r"C:/Users/Bruker/Documents/CASADIPython/casadi-windows-py38-v3.5.5-64bit")
from casadi import *
import casadi as ca
from tabulate import tabulate
from texttable import Texttable
import latextable
from decimal import Decimal

# Call the parameters
import ParameterSOCN
import SimulatorSOCN
#par now represents the dictionary defined in parameter function
par = ParameterSOCN.Params_6wells()




import pandas as pd
#Retrieve initial guesses for the differential states(x0), algebraic states(z0) and
#controlled variables(u0). Data listed in excel, comma separated files.
x0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/x06Sep.csv',header=None).values.reshape(-1)
z0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/z06Sep.csv',header=None).values.reshape(-1)
u0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/u06Sep.csv',header=None).values.reshape(-1)

#Retrieve the lower and upper bounds for the differential states(x), algebraic states(z) and
#controlled variables(u). Data listed in excel, comma separated files.
lbx = pd.read_csv('DatafolderSOCProdGasLimitWhN5/lbx6Sep.csv',header=None).values.reshape(-1)
lbz = pd.read_csv('DatafolderSOCProdGasLimitWhN5/lbz6Sep.csv',header=None).values.reshape(-1)
lbu = pd.read_csv('DatafolderSOCProdGasLimitWhN5/lbu6Sep.csv',header=None).values.reshape(-1)
ubx = pd.read_csv('DatafolderSOCProdGasLimitWhN5/ubx6Sep.csv',header=None).values.reshape(-1)
ubz = pd.read_csv('DatafolderSOCProdGasLimitWhN5/ubz6Sep.csv',header=None).values.reshape(-1)
ubu = pd.read_csv('DatafolderSOCProdGasLimitWhN5/ubu6Sep.csv',header=None).values.reshape(-1)

x0O = x0
z0O = z0
u0O = u0



#Define the parameter intial values(constant, if not manually changed)
p0 = ca.vertcat(par['GOR'],par['wmax_gl'],par['wmax_pg'],par['Powmax_glcom'],par['p_go'],par['p_oo'],par['omega1'],par['omega2'],par['omega3']) 
p0Real = [0.125  , 0.13 , 0.13, 0.14, 0.125, 0.135, 8, 10, 19, 20, 18, 20, 20, 20]
ep = 1e-8


#Function for perturbing manipulated variable and disturbance at same time
def OptimizerDiffud(valve, disturbance, eps1, eps2):
    #Retrieve return variables of the integrator function
    F,x_var, z_var, u_var, p_var, alg, dif, L, g_var =  SimulatorSOCN.CentralizedSimulator_F(par)
    #Change in openings
    ##Change GLC##
    ubu1 = []
    lbu1 = []
    dist = []
    for c in u0: #Make local variables and keep all other manipulated constant
        lbu1.append(c)
    for d in u0:
        ubu1.append(d)
    for e in p0Real:
        dist.append(e)
        
    GLC2case = []
    lbufunc = lbu1
    ubufunc = ubu1
    
    
    
    #perturb the disturbance and valve opening
    Delta_d = dist[disturbance]*eps2
    Delta_u = u0[valve]*eps1
    
    constu = u0[valve]*(1+eps1) #Pretrubation of valve opening for finite difference
    constd = dist[disturbance]*(1+eps2) #Pretrubation of disturbance opening for finite difference
    
    #if eps != 0: #If change is 0 we want nominal bounds
    lbufunc[valve] = constu
    ubufunc[valve] = constu
    dist[disturbance] = constd
   
    
    ################### Optimizer ###################
     
    #Get shape of controlled, and differential states
    nu = u_var.shape[0]
    nx = x_var.shape[0]
    
    #Define equality constraints of nlp, equals the model equations
    eqcon = ca.vertcat(alg, dif) 
    x = ca.vertcat(u_var, x_var, z_var)
    
    #Define the optimization problem(x = states, L= objective function, g = inequality constraints, p = parameters)
    nlp = {
           'x': ca.vertcat(u_var, x_var, z_var),
           'f': L,
           'g': ca.vertcat(eqcon, g_var),
           'p': p_var,
           }
    
    #Define upper/lower bounds for the inequality constraints
    lbg = ca.vertcat(np.full(eqcon.shape, 0), np.full(g_var.shape, -ca.inf))
    ubg = ca.vertcat(np.full(eqcon.shape, 0), np.full(g_var.shape, 0))
    
    #Use IPOPT(interior point optimizer) fromt the CasADI framework to solve the nlp
    opt_inst = ca.nlpsol('opt_inst', 'ipopt', nlp)
    
    #Extract the solution of the optimization, feed ipopt initial values, lower and upper bounds
    opt_res = (opt_inst(p= dist,x0 = ca.vertcat(u0,x0,z0), lbx = ca.vertcat(lbufunc,lbx,lbz), ubx = ca.vertcat(ubufunc,ubx,ubz), lbg=lbg, ubg=ubg))
    #u0,x0,z0
    states = opt_res['x']
    cost = opt_res['f']
    
    
    return cost, Delta_u, Delta_d



