
import numpy as np 
from sys import path
path.append(r"C:/Users/Bruker/Documents/CASADIPython/casadi-windows-py38-v3.5.5-64bit")
from casadi import *
import casadi as ca
from tabulate import tabulate
from texttable import Texttable
import latextable
from decimal import Decimal
import xlsxwriter

# Call the parameters
import ParameterSOCN
import SimulatorSOCN
#par now represents the dictionary defined in parameter function
par = ParameterSOCN.Params_6wells()




import pandas as pd
#Retrieve initial guesses for the differential states(x0), algebraic states(z0) and
#controlled variables(u0). Data listed in excel, comma separated files.
x0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/x06Sep.csv',header=None).values.reshape(-1)
z0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/z06Sep.csv',header=None).values.reshape(-1)
u0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/u06Sep.csv',header=None).values.reshape(-1)

#Retrieve the lower and upper bounds for the differential states(x), algebraic states(z) and
#controlled variables(u). Data listed in excel, comma separated files.
lbx = pd.read_csv('DatafolderSOCProdGasLimitWhN5/lbx6Sep.csv',header=None).values.reshape(-1)
lbz = pd.read_csv('DatafolderSOCProdGasLimitWhN5/lbz6Sep.csv',header=None).values.reshape(-1)
lbu = pd.read_csv('DatafolderSOCProdGasLimitWhN5/lbu6Sep.csv',header=None).values.reshape(-1)
ubx = pd.read_csv('DatafolderSOCProdGasLimitWhN5/ubx6Sep.csv',header=None).values.reshape(-1)
ubz = pd.read_csv('DatafolderSOCProdGasLimitWhN5/ubz6Sep.csv',header=None).values.reshape(-1)
ubu = pd.read_csv('DatafolderSOCProdGasLimitWhN5/ubu6Sep.csv',header=None).values.reshape(-1)


#Define the parameter intial values(constant, if not manually changed)
p0 = ca.vertcat(par['GOR'],par['wmax_gl'],par['wmax_pg'],par['Powmax_glcom'],
                par['p_go'],par['p_oo'],par['omega1'],par['omega2'],par['omega3']) 



ep = 1e-8 #perturbation factor

x_store = []
z_store = []
u_store = []

t_span = np.arange(40000)


#Return how the CV's change with perturbation for the disturbances
def Optimizer(disturbance, eps):
    #Retrieve return variables of the integrator function
    F,x_var, z_var, u_var, p_var, alg, dif, L, g_var =  SimulatorSOCN.CentralizedSimulator_F(par)
    
    #Retrieve the initial data
    x0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/x06Sep.csv',header=None).values.reshape(-1)
    z0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/z06Sep.csv',header=None).values.reshape(-1)
    u0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/u06Sep.csv',header=None).values.reshape(-1)
    
    #Define the parameter values locally
    p0Real = [0.125  , 0.13  , 0.13 , 0.14 , 0.125 , 0.135 , 8, 10, 19, 20, 18, 20, 20, 20] 
    p0Real1 = p0Real
    
    uk = u0
    xf = x0
    zk = z0
    
    GLC2case = []
    
    
    for k in t_span:
        
    
        #Solving the initial value problem 
        inputs = ca.vertcat(uk, p0Real1)
        Fk = F(x0 = xf, z0 = zk, p = inputs) 
        #Retrieving the differential states
        xf = (Fk['xf']).full()
        #Retrieving the algebraic states
        zk = (Fk['zf']).full()
        
        zk[79] = 10 #make sure the constraint is active
        #Append results
        x_store.append(xf)
        u_store.append(uk)
        z_store.append(zk)
        #Perturb the disturbances
        if k == 1:
            diff = p0Real1[disturbance]*eps
            p0Real1[disturbance] = p0Real1[disturbance] + diff
        
    GLC2case.append(zk[7]) #Wellhead pressure 2   1
    GLC2case.append(zk[1]) #Annulus pressure 2    2
    GLC2case.append(zk[19]) #Bottomhole pressure 2  3
    GLC2case.append(zk[11]) #Wellhead pressure 6   4
    GLC2case.append(zk[5]) #Annulus pressure 6    5
    GLC2case.append(zk[23]) #Bottomhole pressure 6   6
    GLC2case.append(zk[74]) #Manifold pressure   7
    GLC2case.append(xf[29]) #Discharge pressure comp 3   8
    GLC2case.append(xf[20]) #Separator pressure   9
    GLC2case.append(zk[55]) #Oil flow 2   10
    GLC2case.append(zk[49]) #Gas flow 2   11
    GLC2case.append(zk[59]) #Oil flow 6   12
    GLC2case.append(zk[53]) #Gas flow 6   13
    GLC2case.append(zk[78]) #Oil out separator  14
    GLC2case.append(zk[79]) #Produced gas   15
    GLC2case.append(zk[107]) #Tot gaslift  16
    
    return GLC2case,diff
    
#Use finite difference to obtain the how the CV's change with perturbing the disturbances
def GetGyd(disturbance1, disturbance2, eps):
    #finite difference for first disturbance
    Delta_y1 = []
    Delta_y2 = []
    Listu1, Delta_u1 = Optimizer(disturbance1, eps)
    Listu2, Delta_u2 = Optimizer(disturbance2, eps)
    ListNom, Delta_nom = Optimizer(disturbance1, 0)
    for i in range(len(Listu1)):
        Delta_y1.append((Listu1[i][0] - ListNom[i][0])/Delta_u1)
    for i in range(len(Listu2)):
        Delta_y2.append((Listu2[i][0] - ListNom[i][0])/Delta_u2)
        
    return Delta_y1, Delta_y2
    

#From the data, get the Gyd matrix on correct form 
def GetGydMatrix(disturbance1, disturbance2, eps):
    
    GyMat = np.zeros((16,2)) #16
    array11, array22 = GetGyd(disturbance1,disturbance2,eps)
    #Reshape arrays 
    array1 = np.array(array11)
    array2 = np.array(array22)
    array1 = array1.reshape((16,1)) #16
    array2 = array2.reshape((16,1)) #16
    
    GyMat[:, 0] = array1[:, 0]
    GyMat[:, 1] = array2[:, 0]
    
    return GyMat
    
#Retireve Gymat   
matd = GetGydMatrix(1,5,ep) 


workbook = xlsxwriter.Workbook('DataForBandB/Gyd.xlsx')
 
#Add workbook to worksheet
worksheet = workbook.add_worksheet()
 
# Use the worksheet object to write
# data via the write() method.

row = 0
col = 0
 
# Iterate over the data and write it out row by row.
for d1, d2 in (matd):
    worksheet.write(row, col, d1)
    worksheet.write(row, col + 1, d2)
    row += 1
 
workbook.close()
# Close the Excel file via the close() method.
