#Parameter file
#Function returns a dictionary with the models constant parameters

import numpy as np

def Params_6wells():
    par = {}  #Dictionary to store the parameters
    par['n_w'] = 6 #Number of wells
        
    ##### Well Parameters ####
    
    #Length, height and diameter of wells[m]
    par['L_w'] = np.array([1500, 1500, 1500, 1500, 1500, 1500])
    par['H_w'] = np.array([1000,1000,1000,1000,1000,1000])
    par['D_w'] = np.array([0.121,0.121,0.121,0.121,0.121,0.121])
    
    #Length, height and diameter of bottom hole[m]
    par['L_bh'] = np.array([500, 500, 500, 500, 500, 500])
    par['H_bh'] = np.array([500, 500, 500, 500, 500, 500])
    par['D_bh'] = np.array([0.121,0.121,0.121,0.121,0.121,0.121])
    
    #Length, height and diameter of annuluses[m]
    par['L_a'] = par['L_w'] # Lenght of annuls equals length of well
    par['H_a'] = par['H_w'] # Height of annuls equals length of well
    par['D_a'] = np.array([0.189, 0.189, 0.189, 0.189, 0.189, 0.189]) 
    
    #Density oil, injection valve char and production choke valve char
    par['rho_o'] = np.array([8,8,7.9,8,8.2,8.05]) *1e2 #[kg/m^3]
    par['C_iv'] = np.array([0.1e-3,0.1e-3,0.1e-3,0.1e-3,0.1e-3,0.1e-3]) #[m^2]
    par['C_pc'] = np.array([2e-3,2e-3,2e-3,2e-3,2e-3,2e-3])  #[m^2]
    
    #Gas-oil ratio of wells[kg/kg], possible disturbance
    #par['GOR'] = np.array([0.1 ,0.12,0.09,0.108,0.115,0.102]) + 0.01
    #par['GOR'] = np.array([0.11 ,0.12 ,0.12,0.12,0.12,0.13]) + 0.0329
    #par['GOR'] = np.array([0.11  ,0.115 + 0.02 ,0.125  ,0.11,0.132,0.13]) + 0.0329 #Manages + 0.025
    #par['GOR'] = np.array([0.11  ,0.115  ,0.125   ,0.111,0.131,0.13]) + 0.0329 #Manages + 0.055 
    #par['GOR'] = np.array([0.13  ,0.12  ,0.13  , 0.12, 0.13, 0.12]) 
    #par['GOR'] = np.array([0.13  ,0.12 + 0.002875 * 4   ,0.13  , 0.115, 0.14, 0.125]) + 0.0279
    #par['GOR'] = np.array([0.13  ,0.122    ,0.135  , 0.115 - 0.002875 * 4 , 0.127, 0.125]) + 0.027
    #par['GOR'] = np.array([0.133  ,0.128   ,0.135  , 0.115 + 0.002875 * 4 , 0.132, 0.13]) + 0.0263
    #par['GOR'] = np.array([0.133  ,0.128   ,0.135  , 0.125  , 0.132, 0.13]) + 0.0263
    #par['GOR'] = np.array([0.13  ,0.125 + 0.003125 *1   ,0.13  , 0.12  , 0.129, 0.125]) + 0.026
    #par['GOR'] = np.array([0.154  ,0.152   ,0.156   ,0.148 + 0.00379 * 1   ,0.153, 0.154]) #Manages up to 10% increase and 2.5, 5 and 10% decrease 
    #par['GOR'] = np.array([0.154  ,0.152 + 0.00152*3   ,0.156   ,0.1488   ,0.153, 0.154]) #Manages decrease and increase of 2.5, 5 and 10% well 4 + 0.00379*4
    #par['GOR'] = np.array([0.154  ,0.152   ,0.156   ,0.1488 + 0.00379*2   ,0.153, 0.154]) + 0.0028
    par['GOR'] = np.array([0.125    , 0.13  ,0.13  ,0.14   ,0.125  , 0.135 ]) 
    #par['GOR'] = np.array([0.125 , 0.13     ,0.129  ,0.142   ,0.1252 , 0.135]) 
    #4.36397
    #0.00013
    #0.537969
    par['p_res'] = np.array([150,155,155,160,155,155]) #Reservoir pressure[bar]
    par['PI'] = np.array([15,14,15,14,14,15])* 0.5 #Productvity index wells[kg s^-1 bar^-1]
    #par['PI'] = np.array([7,7,7,7,7,7])* 0.5 #Productvity index wells[kg s^-1 bar^-1]
    par['T_a'] = np.array([273, 273, 273, 273, 273, 273]) + 28 #Annulus temperature[K]
    par['T_w'] = np.array([273, 273, 273, 273, 273, 273]) + 32 #Well temperature[K]
    
    #Area of well, bottom hole and volume of annulus
    par['A_w'] = np.pi*(par['D_w']/2)**2 #[m^2]
    par['A_bh'] = np.pi*(par['D_bh']/2)**2 #[m^2]
    par['V_a'] = par['L_a']*(np.pi*(par['D_a']/2)**2 - np.pi*(par['D_w']/2)**2) #[m^3]
    #Volume of annulus will equal the area of the total well and annulus minus the well
    

    #Constraints 
    par['wmax_gl'] = np.array([8]) #Max gas lift
    #par['wmax_pg'] = np.array([10]) #Max produced gas
    par['wmax_pg'] = np.array([10]) #Max produced gas
    #par['Powmax_glcom'] = np.array([19]) #Max power #Nominal
    par['Powmax_glcom'] = np.array([19]) #Max power
    
    #General parameters
    par['R'] = 8.314 #Gas constant [m^3 Pa K^-1 mol^-1]
    par['Mw'] = 20e-3 #Molar weighgt kg/mol
    par['tf'] = 1 #Simulation time
    par['mu_oil'] = 0.001 #Oil viscosity[kg m^−1 s^−1]
    
    
    #### Riser System ####
    par['L_r'] = 500 #Length of riser[m]
    par['H_r'] = 500 #Height of riser[m]
    par['D_r'] = 0.121 #Diameter of riser[m]
    A_r = np.pi*(par['D_r']/2)**2 
    par['A_r'] = A_r #Area of riser[m^2]
    par['T_r'] = 30+273 #Temperature riser[K]
    par['C_pr'] = 0.003 #Valve char riser valve[m^2]
    rho_ro = np.sum(par['rho_o'])/6 
    par['rho_ro'] = rho_ro #Density of oil in riser[kg/m^3]
    
    
    #### Separator ####
    par['L_s'] = 5 #10length Separator[m] #Oversized
    par['r_s'] = 1.65 #radius Separator[m] #Oversized
    par['T_s'] = 29 + 273 #Temperature Separator[K]
    V_sep = np.pi * par['r_s']**2 * par['L_s']
    par['V_s'] = V_sep #Volume of Separator[m3]
    par['C_gs'] = 5.5*0.001 #Valve char gas outlet[m^2]
    par['C_os'] = 5.5*0.001*0.5*0.5#5.5*0.001*0.5*0.5 #Valve char oil outlet[m^2]
    par['p_go'] = 20 #pressure gas out[bar]
    par['p_oo'] = 20 - 2 #pressure oil out[bar]
    
    #### Compressors ####
    par['n_c'] = 1  #This parameter is just an early implementation error.
    par['T_d'] = 298 #Temperature out of compessor(assume heat is removed)[K]
    par['C_in'] = 9e-4*2.93#Valve char is equal for all in/out valves[m^2]
    par['T_in'] = 298 #Temperature inlet compressors[K]
    par['Z_in'] = 0.9 #Compression factor(difference from ideal behaviour)[-]
    par['n_v'] = 1.27 #Polytropic coefficient[-]
    par['C_out'] = 1.201e-3 #Valve char out, not used at this impelementation[m^2]
    par['C_rec'] = 1.1*3.5e-5 *2 #Recycle valve char[m^2]
    par['omega1'] = 20
    par['omega2'] = 20
    par['omega3'] = 20
    #alpha values for the approximation of pressure ratio
    par['alpha_1'] = 1.05 * 0.745 *2.3
    par['alpha_2'] = 0.7 * -1.4e-2 *-1
    par['alpha_3'] = 0.3 * 0.11 * -4.09e-2 
    par['alpha_4'] = 1.75 * 0.13* 9.86e-4
    par['alpha_5'] = 1.0 * 0.5* -4.25e-4 *-1
    par['alpha_6'] = 300* (-0.15)* 2.45e-5 *2
    
    #beta values for the approximation of efficiency
    par['beta_1'] = 0.7* 9*5.91e-2 *200
    par['beta_2'] = -2.13e-1 *2
    par['beta_3'] = 2.93e-1
    par['beta_4'] = 2.97e-3
    par['beta_5'] = -2.68e-5 
    par['beta_6'] = -1.1e1 *(-0.1)*1.2 *2
    
    #Dynamic coefficients for compressor dynamic equations
    par['Coef_1'] = 1e4
    par['Coef_2'] = 1e5
    par['Coef_3'] = 1
    
    a = 0.1#0.073
    #gamma values for further implementation of surge and choke constraints 
    #Comp1
    par['gamma_11'] = 250*0.015* 0.55 * a#0.085#0.085 #0.08
    par['gamma_21'] = 0.5* 3.812e-2 * 0.1
    par['gamma_31'] = 1.08* 0.615* 3 * 0.7
    
    #Comp2
    #gamma values for further implementation of surge and choke constraints
    par['gamma_12'] = 250*0.015* 0.55 * a#0.085#0.085 #0.06
    par['gamma_22'] = 0.5* 3.812e-2 * 0.1
    par['gamma_32'] = 1.08* 0.615* 3 * 0.7
    
    #Comp3
    #gamma values for further implementation of surge and choke constraints
    par['gamma_13'] = 250*0.015* 0.55 * a#0.085#0.085#0.09  #0.058
    par['gamma_23'] = 0.5* 3.812e-2 * 0.1
    par['gamma_33'] = 1.08* 0.615* 3 * 0.7
    
    #### Gas lift ####
    par['L_gl'] = 500 #Length gas lift line[m]
    par['r_gl'] = 0.15 #radius gas lift line[m]
    par['C_gl'] = np.array([5e-5,5e-5,5e-5,5e-5,5e-5,5e-5]) #Valve char gas lift valves[m^2]
    par['C_iv'] = np.array([0.1e-3,0.1e-3,0.1e-3,0.1e-3,0.1e-3,0.1e-3]) * 1.35  #Valve char injection valves[m^2]
    
    return par
