#Simulation file
#Constructs the integrator


import numpy as np
from casadi import *

def CentralizedSimulator_F(par):
    
    ## Retriving the parameters from Param function ##
    
    #Wells
    n_w = par['n_w'] 
    L_w = par['L_w']
    H_w = par['H_w']
    D_w = par['D_w']
    L_bh = par['L_bh']
    H_bh = par['H_bh']
    D_bh = par['D_bh']
    L_a = par['L_a']
    H_a = par['H_a']
    D_a = par['D_a']
    rho_o = par['rho_o']
    C_iv = par['C_iv']
    C_pc = par['C_pc']
    mu_oil = par['mu_oil']
    A_w = par['A_w']
    A_bh = par['A_bh']
    V_a = par['V_a']
    p_res = MX.sym('p_res',n_w)
    PI = MX.sym('PI',n_w) 
    T_a = MX.sym('T_a',n_w)
    T_w = MX.sym('T_w',n_w)
    R = par['R']
    Mw = par['Mw']
    
    #Riser
    T_r = par['T_r']
    L_r = par['L_r']
    A_r = par['A_r']
    H_r = par['H_r']
    D_r = par['D_r']
    C_pr = par['C_pr']
    rho_ro = par['rho_ro']
    
    #Separator
    L_s = par['L_s']
    r_s = par['r_s']
    T_s = par['T_s']
    C_gs = par['C_gs']
    C_os = par['C_os']
    v_s = par['V_s']
   
    
    #Compressor
    n_c = par['n_c']
    C_in = par['C_in']
    C_out = par['C_out']
    C_rec = par['C_rec']
    T_in = par['T_in']
    T_d = par['T_d']
    Z_in = par['Z_in']
    n_v = par['n_v']
    #alphas 
    alpha_1 = par['alpha_1']
    alpha_2 = par['alpha_2']
    alpha_3 = par['alpha_3']
    alpha_4 = par['alpha_4']
    alpha_5 = par['alpha_5']
    alpha_6 = par['alpha_6']
    #beta
    beta_1 = par['beta_1']
    beta_2 = par['beta_2']
    beta_3 = par['beta_3']
    beta_4 = par['beta_4']
    beta_5 = par['beta_5']
    beta_6 = par['beta_6']
    #gammas1(Further implementation)
    gamma_11 = par['gamma_11']
    gamma_21 = par['gamma_21']
    gamma_31 = par['gamma_31'] 
    #gammas2(Further implementation)
    gamma_12 = par['gamma_12']
    gamma_22 = par['gamma_22']
    gamma_32 = par['gamma_32'] 
    #gammas3(Further implementation)
    gamma_13 = par['gamma_13']
    gamma_23 = par['gamma_23']
    gamma_33 = par['gamma_33'] 
    #Dynamic Coefficients for compressor dynamic equations
    Coef_1 = par['Coef_1']
    Coef_2 = par['Coef_2']
    Coef_3 = par['Coef_3']
    
    #Gaslift
    C_iv = par['C_iv']
    C_gl = par['C_gl']
    L_gl = par['L_gl']
    r_gl = par['r_gl']


    #Differential states 
    #Well system         
    m_ga = MX.sym('m_ga',n_w) #Mass gas in annulus[ton] (23-28) x
    m_gt = MX.sym('m_gt',n_w) #Mass gas in tubing[ton]  (29-34) x
    m_ot = MX.sym('m_ot',n_w) #Mass oil in tubing[ton] (35-40) x
    #Riser
    m_gr = MX.sym('m_gr',1) #Mass gas in riser[ton] (41) x
    m_or = MX.sym('m_or',1) #Mass oil in riser[ton] (42) x
    #Separator
    p_gs = MX.sym('p_gs',1) #Pressure of gas in separator[bar] (43)
    h_ls = MX.sym('h_ls',1) #Height of oil in separator[bar] (44) #NO SOC x
    #Compressor 1
    p_s1 = MX.sym('p_s1',n_c) #Suction Pressure Gas lift Compressor 1[bar] (45)x
    p_d1 = MX.sym('p_d1',n_c) #Discharge Pressure Gaslift Compressor 1[bar] (46)x
    w_c1 = MX.sym('w_c1',n_c) #Gas massflow rate Gas-lift Compressor 1[kg/s] (47)x
    #Compressor 2
    p_s2 = MX.sym('p_s2',n_c) #Suction Pressure Gas lift Compressor 2[bar] (48) x
    p_d2 = MX.sym('p_d2',n_c) #Discharge Pressure Gas lift Compressor 2[bar] (49) x
    w_c2 = MX.sym('w_c2',n_c) #Gas massflow rate Gas lift Compressor 2[kg/s] (50) x
    #Compressor 3
    p_s3 = MX.sym('p_s3',n_c) #Suction Pressure Gas lift Compressor 3[bar] (51) x
    p_d3 = MX.sym('p_d3',n_c) #Discharge Pressure of Gas lift Compressor 3[bar] (52) 
    w_c3 = MX.sym('w_c3',n_c) #Gas massflow rate in Gas-lift Compressor 3[kg/s] (53) x
    #Gas Lift
    m_gl = MX.sym('m_gl',1) #Mas gas in gas line[ton] (54)x
    
    
    #Algebraic states
    #Well
    p_ai = MX.sym('p_ai',n_w) #Annulus pressure at injection[bar] (55-60)
    p_wh = MX.sym('p_wh',n_w) #Wellhead pressure[bar] (61-66)
    p_wi = MX.sym('p_wi',n_w) #Injection point pressure in tubing[bar] (67-72) x
    p_bh = MX.sym('p_bh',n_w) #Bottom-hole pressure[bar] (73-78)
    rho_ai = MX.sym('rho_ai',n_w) #Density of gas annulus injection point[bar] (79-84) x
    rho_m = MX.sym('rho_m',n_w) #Density mixed oil/gas in tubing[kg/m^3] (85-90) x
    w_iv = MX.sym('w_iv',n_w) #Flow gas through injection valve[kg/s] (91-96) x
    w_pc = MX.sym('w_pc',n_w) #Flow through production choke[kg/s] (97-102) #NO SOC x
    w_pg = MX.sym('w_pg',n_w) #Flow gas through production choke[kg/s] (103-108) #NO SOC
    w_po = MX.sym('w_po',n_w) #Flow oil through production choke[kg/s] (109-114) #NO SOC
    w_ro = MX.sym('w_ro',n_w) #Flow oil from reservoir[kg/s] (115-120) #NO SOC x
    w_rg = MX.sym('w_rg',n_w) ##Flow gas from reservoir[kg/s] (121-126) #NO SOC x
    #Riser 
    p_rh = MX.sym('p_rh', 1) #Pressure riser head[bar] (127) x
    rho_r = MX.sym('rho_r',1) #density oil/gas riser[kg/m^3] (128) x
    p_m = MX.sym('p_m', 1) #Manifold pressure[bar] (129)
    w_pr = MX.sym('w_pr', 1) #Flow through riser valve[kg/s] (130) #NO SOC x
    w_to = MX.sym('w_to', 1) #Flow oil through riser valve[kg/s] (131) #NO SOC x
    w_tg = MX.sym('w_tg', 1) #Flow gas through riser valve[kg/s] (132) #NO SOC x
    #Separator
    w_os = MX.sym('w_os',1) #Produced oil out of separator[kg/s] (133) #NO SOCx
    w_gs = MX.sym('w_gs',1) #Produced gas out of separator[kg/s] (134) x
    rho_gs = MX.sym('rho_gs', 1) #Gas density in separator[kg/m^3] (135) x
    p_os = MX.sym('p_os', 1) #Separator oil pressure[bar] (136)x
    v_os = MX.sym('v_os', 1) #Volume of oil in separator[m^3] (137)x
    v_gs = MX.sym('v_gs',1) #Volume of gas in separator[m^3] (138)x
    #Compressor 1
    w_in1 = MX.sym('w_in1',n_c) #Flow gas in compressor 1[kg/s] (139)x
    w_out1 = MX.sym('w_out1',n_c) #Flow gas out compressor 1[kg/s] (140) #NO SOCx
    rho_in1 = MX.sym('rho_in1',n_c) #Density gas in compressor 1[kg/m^3] (141) x
    rho_d1 = MX.sym('rho_d1',n_c) #Density gas out compressor 1[kg/m^3] (142) x
    Phi1 = MX.sym('Phi1',n_c) #Pressure Ratio compresor 1[-] (143)x
    Pow1 = MX.sym('Pow1',n_c) #Power consumption compressor 1[kW] (144)x
    y_p1 = MX.sym('y_p1',n_c) #Polytropic Head compressor 1[m] (145)x
    n_p1 = MX.sym('n_p1',n_c)  #Polytropic Efficiency 1[%] (146)x
    w_rec1 = MX.sym('w_rec1', n_c) #Recycle mass flow[kg/s] (147) #NO SOCx
    #Further implementation
    Phi_max1 = MX.sym('Phi_max1',n_c) #Max Pressure ratio (148) #NO SOCx
    gamma_2_dummy1 = MX.sym('gamma_2_dummy1',n_c) #constraint (149) #NO SOCx
    #Compressor 2
    w_in2 = MX.sym('w_in2',n_c) #Flow gas in compressor 2[kg/s] (150) #NO SOCx
    w_out2 = MX.sym('w_out2',n_c) #Flow gas out compressor 2[kg/s] (151) #NO SOCx
    rho_in2 = MX.sym('rho_in2',n_c) #Density gas in compressor 2[kg/m^3] (152)x
    rho_d2 = MX.sym('rho_d2',n_c) #Density gas out compressor 2[kg/m^3] (153)x
    Phi2 = MX.sym('Phi2',n_c) #Pressure Ratio compresor 2[-] (154)x
    Pow2 = MX.sym('Pow2',n_c) #Power consumption compressor 2[kW] (155)x
    y_p2 = MX.sym('y_p2',n_c) #Polytropic Head compressor 2[m] (156)x
    n_p2 = MX.sym('n_p2',n_c) #Polytropic Efficiency 2[%] (157)x
    w_rec2 = MX.sym('w_rec2', n_c) #Recycle mass flow 2[kg/s] (158) #NO SOCx
    #Further implementation
    Phi_max2 = MX.sym('Phi_max2',n_c) #Max pressure ratio (159) #NO SOCx
    gamma_2_dummy2 = MX.sym('gamma_2_dummy2',n_c) #constraint (160) #NO SOCx
    #Compressor 3
    w_in3 = MX.sym('w_in3',n_c) #Flow gas in compressor 2[kg/s] (161) #NO SOCx
    w_out3 = MX.sym('w_out3',n_c) #Flow gas out compressor 2[kg/s] (162) #NO SOCx
    rho_in3 = MX.sym('rho_in3',n_c) #Density gas in compressor 2[kg/m^3] (163)x
    rho_d3 = MX.sym('rho_d3',n_c) #Density gas out compressor 2[kg/m^3] (164)x
    Phi3 = MX.sym('Phi3',n_c) #Pressure Ratio compresor 2[-] (165)x
    Pow3 = MX.sym('Pow3',n_c) #Power consumption compressor 2[kW] (166)x
    y_p3 = MX.sym('y_p3',n_c) #Polytropic Head compressor 2[m] (167)x
    n_p3 = MX.sym('n_p3',n_c) #Polytropic Efficiency 2[%] (168)x
    w_rec3 = MX.sym('w_rec3', n_c) #Recycle mass flow 2[kg/s] (169) #NO SOCx
    #Further implementation
    Phi_max3 = MX.sym('Phi_max3',n_c) #Max pressure ratio (170) #NO SOCx
    gamma_2_dummy3 = MX.sym('gamma_2_dummy3',n_c) #constraint (171) #NO SOC x
    #Gl system
    w_gl = MX.sym('w_gl',n_w) #Flow through gas lift choke[kg/s] (172-177)x
    p_out = MX.sym('p_out',1) #Pressure in gas lift line[bar] (178)x
    rho_out = MX.sym('rho_out',1) #density of gas in gas lift line[kg/m^3] (179)x 
    
    
    #Control input
    
    #Gas lift
    u_gl = MX.sym('u_gl', n_w) #Valve opening gas lift chokes[0-1] (0-5)
    
    #Separator
    z_ov = MX.sym('z_ov', 1) #Valve opening separator oil out[0-1] (6)
    
    #Compressor
    u_1 = MX.sym('u_1',n_c) #Valve opening inlet compressor 1[0-1] (7)
    
    #Wells
    u_pc = MX.sym('u_pc', n_w) #Valve opening production chokes[0-1] (8-13)
    
    #Compressor
    u_2 = MX.sym('u_2',n_c) #Valve opening inlet compressor 2, outlet 1[0-1] (14)
    u_3 = MX.sym('u_3',n_c) #Valve opening inlet compressor 3, outlet 2[0-1] (15)
    u_4 = MX.sym('u_4',n_c) #Valve opening outlet compressor 3[0-1] (16)
    u_rec1 = MX.sym('u_rec1',n_c) #Valve opening recycle compressor 1[0-1] (17)
    u_rec2 = MX.sym('u_rec2',n_c) #Valve opening recycle compressor 2[0-1] (18)
    u_rec3 = MX.sym('u_rec3',n_c) #Valve opening recycle compressor 3[0-1] (19)
    omega1 =MX.sym('omega1',n_c) #Speed of compressor 1[rad/s] (20)
    omega2 = MX.sym('omega2',n_c) #Speed of compressor 2[rad/s] (21)
    omega3 = MX.sym('omega2',n_c) #Speed of compressor 3[rad/s] (22)
    
    #Parameters(Possible to introduce more)
    #Wells
    GOR = MX.sym('GOR',n_w)   
    #Separator
    p_go = MX.sym('p_go',1)
    p_oo = MX.sym('p_oo', 1) 

    #Constraints
    wmax_gl = MX.sym('wmax_gl',1)
    wmax_pg = MX.sym('wmax_pg',1)
    Powmax_glcom = MX.sym('Powmax_glcom',1)
    
    
   
    #Algebraic equations
    #Wells                                                 
    f1 = -p_ai*1e5 + ((R*T_a/(V_a*Mw) + 9.81*H_a/V_a)*m_ga*1e3) #Bernoulli                  
    f2 = -p_wh*1e5 + ((R*T_w/Mw)*(m_gt*1e3/(L_w*A_w + L_bh*A_bh - m_ot*1e3/rho_o))) - ((m_gt*1e3+m_ot*1e3 )/(L_w*A_w))*9.81*H_w/2 #Bernoulli             
    f3 = -p_wi*1e5 + (p_wh*1e5 + 9.81/(A_w*L_w)*fmax(0,(m_ot*1e3+m_gt*1e3-rho_o*L_bh*A_bh))*H_w + 128*mu_oil*L_w*w_pc/(3.14*D_w**4*((m_gt*1e3 + m_ot*1e3)*p_wh*1e5*Mw*rho_o)/(m_ot*1e3*p_wh*1e5*Mw + rho_o*R*T_w*m_gt*1e3))) #Bernoulli/Hagen-Poiseuille              
    f4 = -p_bh*1e5 + (p_wi*1e5 + rho_o*9.81*H_bh + 128*mu_oil*L_bh*w_ro/(3.14*D_bh**4*rho_o)) #Bernoulli/Hagen-Poiseuille                   
    f5 = -rho_ai*1e2 +(Mw/(R*T_a)*p_ai*1e5) #Ideal gas law    x             
    f6 = -rho_m*1e2 + ((m_gt*1e3 + m_ot*1e3)*p_wh*1e5*Mw*rho_o)/(m_ot*1e3*p_wh*1e5*Mw + rho_o*R*T_w*m_gt*1e3)#Relationship oil/gas  x        
    f7 = -w_iv + C_iv*sqrt(rho_ai*1e2*fmax(0.001,(p_ai*1e5 - p_wi*1e5)))#Valve equation
    f8 = -w_pc + u_pc*C_pc*sqrt(rho_m*1e2*fmax(0.001,(p_wh*1e5 - p_m*1e5)))#Valve equation
    f9 = -w_pg + ((m_gt*1e3)/fmax(1e-3,(m_gt*1e3+m_ot*1e3)))*w_pc #massfraction of gas
    f10 = -w_po + ((m_ot*1e3)/fmax(1e-3,(m_gt*1e3+m_ot*1e3)))*w_pc #massfraction oil
    f11 = -w_ro + PI*1e-6*(p_res*1e5 - p_bh*1e5)#From definition of Productivity index
    f12 = -w_rg + GOR*w_ro #From definition of Gas-oil ratio
    #Riser system
    f15 = -p_rh*1e5 + ((R*T_r/Mw))*(m_gr*1e3/(L_r*A_r)) - ((m_gr*1e3+m_or*1e3)/(L_r*A_r))*9.81*H_r/2 #Bernoulli
    f16 = -rho_r*1e2 + ((m_gr*1e3 + m_or*1e3)*p_rh*1e5*Mw*rho_ro)/(m_or*1e3*p_rh*1e5*Mw +rho_ro*R*T_r*m_gr*1e3) 
    f17 = -p_m*1e5 + (p_rh*1e5 + 9.81/(A_r*L_r)*(m_or*1e3+m_gr*1e3)*H_r + 128*mu_oil*L_r*w_pr/(np.pi*D_r**4*((m_gr*1e3+m_or*1e3) * p_rh*1e5*Mw*rho_ro) / (m_or*1e3*p_rh*1e5*Mw+rho_ro*R*T_r*m_gr*1e3))) #Realationship oil/gas
    f18 = -w_pr + 1*C_pr * np.sqrt(rho_r*1e2*fmax(0.001,(p_rh*1e5-p_gs*1e5))) #Valve equation
    f19 = -w_to + (m_or*1e3/(m_gr*1e3 + m_or*1e3))*w_pr #massfraction oil
    f20 = -w_tg + (m_gr*1e3/(m_gr*1e3 + m_or*1e3))*w_pr #massfraction gas
    #Separator system
    f21 = -w_os + z_ov*C_os*sqrt(rho_ro*1e2*fmax(0.001,(p_os*1e5 - p_oo*1e5))) #Valve equation 
    f22 = -w_gs + C_gs*np.sqrt(rho_gs*1e2 *fmax(0.001,(p_gs*1e5 - p_go*1e5))) #Valve equation
    f23 = -rho_gs*1e2 + (Mw/(T_s * R) * p_gs*1e5) #Ideal gas law x
    f24 = -p_os*1e5 + p_gs*1e5 + rho_ro * 9.81 * h_ls #Bernoulli equation
    f25 = -v_os  + ((0.5*r_s**2)*((2*np.arccos(fmax(0,(r_s-h_ls)/r_s)))-np.sin((2*np.arccos(fmax(0,(r_s-h_ls)/r_s))))))*L_s #Based on equation of segment/circle, derivative of the Area
    f26 = -v_gs  + fmax(0,(v_s - v_os)) #Based on relation volume of gas/oil in separator
    #Compressor 1
    f27 = -w_in1 + C_in*u_1*np.sqrt(rho_in1*1e2*fmax(0.001,(p_gs*1e5 - p_s1*1e5)))#Valve equation x
    f28 = -w_out1 + C_in*u_2*np.sqrt(rho_d1*1e2*fmax(0.001,(p_d1*1e5-p_s2*1e5)))#Valve equation x
    f29 = -rho_in1*1e2 + (Mw/(R*T_in)*p_gs*1e5)#Ideal gas law x
    f30 = -rho_d1*1e2 + (Mw/(R*T_d)*p_d1*1e5)#Ideal gas law x
    f31 = -Phi1 + alpha_1 + alpha_2*omega1 + alpha_3*w_c1 + alpha_4*omega1*w_c1 + alpha_5*omega1*omega1 + alpha_6*w_c1*w_c1# xPolynomial realationship/approximation
    f32 = -Pow1 + (y_p1/(n_p1))*w_c1#Based on how much of the potential power that can be utilized x
    f33 = -y_p1*1e5 + (Z_in *R *T_in/(Mw))*(n_v/(n_v-1)) *((Phi1**((n_v-1)/n_v))-1)# xEquation for polytropic head
    f34 = -n_p1*1e2 + beta_1 + beta_2*omega1 + beta_3*Phi1 + beta_4*omega1*Phi1 + beta_5*omega1*omega1 + beta_6*Phi1*Phi1# xPolynomial realationship/approximation
    f35 = -Phi_max1 + gamma_11*(w_c1-gamma_21) + gamma_31 #Further work
    f36 = - gamma_2_dummy1 + w_c1 - ((Phi1 - gamma_31)/gamma_11) #Further work
    f37 = -w_rec1 + C_rec*u_rec1*np.sqrt(rho_d1*1e2*fmax(0.0001,(p_d1*1e5 - p_s1*1e5))) #Valve equation x
    #Compressor 2
    f38 = -w_in2 + C_in*u_2*np.sqrt(rho_in2*1e2*fmax(0.001,(p_d1*1e5 - p_s2*1e5)))#Valve equation
    f39 = -w_out2 + C_in*u_3*np.sqrt(rho_d2*1e2*fmax(0.001,(p_d2*1e5-p_s3*1e5)))#Valve equation
    f40 = -rho_in2*1e2 + (Mw/(R*T_in)*p_d1*1e5)#Ideal gas law x
    f41 = -rho_d2*1e2 + (Mw/(R*T_d)*p_d2*1e5)#Ideal gas law
    f42 = -Phi2 + alpha_1 + alpha_2*omega2 + alpha_3*w_c2 + alpha_4*omega2*w_c2 + alpha_5*omega2*omega2 + alpha_6*w_c2*w_c2#Polynomial realationship/approximation
    f43 = -Pow2 + (y_p2/n_p2)*w_c2#Based on how much of the potential power that can be utilized
    f44 = -y_p2*1e5 + (Z_in *R *T_in/(Mw))*(n_v/(n_v-1)) *((Phi2**((n_v-1)/n_v))-1)#Equation for polytropic head
    f45 = -n_p2*1e2 + beta_1 + beta_2*omega2 + beta_3*Phi2 + beta_4*omega2*Phi2 + beta_5*omega2*omega2 + beta_6*Phi2*Phi2#Polynomial realationship/approximation
    f46 = -Phi_max2 + gamma_12*(w_c2-gamma_22) + gamma_32 #Further work
    f47 = - gamma_2_dummy2 + w_c2 - ((Phi2 - gamma_32)/gamma_12) #Further work
    f48 = -w_rec2 + C_rec*u_rec2*np.sqrt(rho_d2*1e2*fmax(0.0001,(p_d2*1e5 - p_s2*1e5)))#Valve equation
    #Compressor 3
    f49 = -w_in3 + C_in*u_3*np.sqrt(rho_in3*1e2*fmax(0.001,(p_d2*1e5 - p_s3*1e5)))#Valve equation
    f50 = -w_out3 + C_in*u_4*np.sqrt(rho_d3*1e2*fmax(0.001,(p_d3*1e5-p_out*1e5)))#Valve equation
    f51 = -rho_in3*1e2 + (Mw/(R*T_in)*p_d2*1e5)#Ideal gas law
    f52 = -rho_d3*1e2 + (Mw/(R*T_d)*p_d3*1e5)#Ideal gas law
    f53 = -Phi3 + alpha_1 + alpha_2*omega3 + alpha_3*w_c3 + alpha_4*omega3*w_c3 + alpha_5*omega3*omega3 + alpha_6*w_c3*w_c3#Polynomial realationship/approximation
    f54 = -Pow3 + (y_p3/n_p3)*w_c3#Based on how much of the potential power that can be utilized
    f55 = -y_p3*1e5 + (Z_in *R *T_in/(Mw))*(n_v/(n_v-1)) *((Phi3**((n_v-1)/n_v))-1)#Equation for polytropic head
    f56 = -n_p3*1e2 + beta_1 + beta_2*omega3 + beta_3*Phi3 + beta_4*omega3*Phi3 + beta_5*omega3*omega3 + beta_6*Phi3*Phi3#Polynomial realationship/approximation
    f57 = -Phi_max3 + gamma_13*(w_c3-gamma_23) + gamma_33 #Further work
    f58 = - gamma_2_dummy3 + w_c3 - ((Phi3 - gamma_33)/gamma_13) #Further work
    f59 = -w_rec3 + C_rec*u_rec3*np.sqrt(rho_d3*1e2*fmax(0.001,(p_d3*1e5 - p_s3*1e5)))#Valve equation
    #Gas Lift
    f60 = -w_gl + C_gl*u_gl*np.sqrt(rho_out*1e2*fmax(0.001,(p_out*1e5 - p_ai*1e5)))#Valve equation
    f61 = -p_out*1e5 + R*T_d*m_gl*1e3/(Mw*np.pi*r_gl*r_gl*L_gl)#Ideal gas law x
    f62 = -rho_out*1e2 + (Mw/(R*T_d)*p_out*1e5)#Ideal gas law
    
    #Differential equations 
    #Wells
    df1 = (w_gl - w_iv)*1e-3 #m_ga, massbalance of gas annulus
    df2 = (w_iv + w_rg - w_pg)*1e-3 #m_tg, massbalance of gas tubing
    df3 = (w_ro - w_po)*1e-3 #m_to, massbalance of oil tubing
    #Riser 
    df4 = (sum(w_pg.nz) - w_tg)*1e-3#*1e-3 #m_gt, massbalance gas riser
    df5 = (sum(w_po.nz) - w_to)*1e-3#*1e-3 #m_ot, massbalance oil riser
    #Separator 
    df6 = ((R*T_s/(v_gs*Mw))*(w_tg - w_gs - w_in1)) + (p_gs/(v_gs*rho_ro))*((w_to - w_os)*1e-4)# p_gs, based on derivative of ideal gas law
    df7 = (((w_to - w_os))/rho_ro)/(2*L_s * np.sqrt(h_ls* fmax(0,((2 *r_s)-h_ls))))#h_ls, based on equation of segment/circle, derivative of the Area
    #Compressor system(diff equations)
    #Compressor 1
    df8 = (w_in1  - w_c1 + w_rec1) * Coef_1 #p_s1, based on gas in/out of system
    df9 = (w_c1 - w_out1 - w_rec1) *Coef_2 #p_d1, based on gas in/out of system
    df10 = (p_s1*Phi1 - p_d1) * Coef_3 #w_c1, based on pressure difference between in/out
    # Define variables for combined systems (needed only for decomposition case)
    #Compressor 2
    df11 = (w_in2  - w_c2 + w_rec2) * Coef_1 #p_s2, based on gas in/out of system
    df12 = (w_c2 - w_out2 - w_rec2) *Coef_2 #p_d2, based on gas in/out of system
    df13 = (p_s2*Phi2 - p_d2) * Coef_3 #w_c2, based on pressure difference between in/out
    #Compressor 3
    df14 = (w_in3  - w_c3 + w_rec3) * Coef_1#p_s3, based on gas in/out of system
    df15 = (w_c3 - w_out3 - w_rec3) *Coef_2#p_d3, based on gas in/out of system
    df16 = (p_s3*Phi3 - p_d3) * Coef_3#w_c3, based on pressure difference between in/out
    #Gas lift(diff equations)
    df17 = (w_out3 - sum(w_gl.nz))*1e-3 #m_gl, based on massbalance
                                               
    #Form the DAE system
    dif = vertcat(df1,df2,df3,df4, df5,df6,df7,df8,df9,df10,df11,df12,df13,df14,df15,df16,df17) #Differential equations
    alg = vertcat(f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f15,f16,f17,f18,f19,f20,f21,f22,f23,f24,f25,f26,f27,f28,f29,f30,f31,f32,f33,f34,f35,f36,f37,f38,f39,f40,f41,f42,f43,f44,f45,f46,f47,f48,f49,f50,f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62) #Algebraic equations
    x_var = vertcat(m_ga,m_gt,m_ot,m_gr, m_or,p_gs,h_ls,p_s1,p_d1, w_c1, p_s2,p_d2,w_c2,p_s3,p_d3,w_c3,m_gl) #Differential states
    z_var = vertcat(p_ai,p_wh,p_wi,p_bh,rho_ai,rho_m,w_iv,w_pc,w_pg,w_po,w_ro,w_rg, p_rh,rho_r,p_m,w_pr,w_to,w_tg,w_os,w_gs,rho_gs,p_os,v_os,v_gs,w_in1,w_out1,rho_in1,rho_d1, Phi1,Pow1,y_p1,n_p1,Phi_max1,gamma_2_dummy1,w_rec1,w_in2,w_out2,rho_in2,rho_d2, Phi2,Pow2,y_p2,n_p2,Phi_max2,gamma_2_dummy2,w_rec2,w_in3,w_out3,rho_in3,rho_d3,Phi3,Pow3,y_p3,n_p3,Phi_max3,gamma_2_dummy3,w_rec3,w_gl,p_out,rho_out)#Algebraic states
    u_var = vertcat(u_gl,z_ov,u_1,u_pc,u_2,u_3,u_4,u_rec1,u_rec2,u_rec3)#Control variables
    p_var = vertcat(GOR,wmax_gl,wmax_pg,Powmax_glcom,p_go,p_oo,omega1,omega2, omega3)#Parameters/constraints
    
   
    #Inequality constraints
    g_var = vertcat((w_gs-wmax_pg),((Pow1 + Pow2 + Pow3)-Powmax_glcom),(sum(w_gl.nz)- wmax_gl))
    
    #Objective function(Whant to maximize oil pruduction and minimize power consumption)
    L = -0.6*w_os + 0.03*(Pow1 + Pow2 + Pow3) 
    
    #Free variables need to be added
    alg = substitute(alg,p_res,par['p_res'])
    alg = substitute(alg,PI,par['PI'])
    alg = substitute(alg,T_a,par['T_a'])
    alg = substitute(alg,T_w,par['T_w'])

    #Constructing the total DEA system, into CasADI framework
    dae = {'x': x_var,'z': z_var,'p': vertcat(u_var,p_var),'ode': dif,'alg': alg,'quad': L}
    
    #Define integration time
    opts = {'tf': par['tf']}

    #Create IDAS integrator for the DAE system
    F = integrator('F','idas',dae,opts)
    
    #Returns values
    return F,x_var, z_var, u_var, p_var, alg, dif, L, g_var

