
import numpy as np 
from sys import path
path.append(r"C:/Users/Bruker/Documents/CASADIPython/casadi-windows-py38-v3.5.5-64bit")
from casadi import *
import casadi as ca
from tabulate import tabulate
from texttable import Texttable
import latextable
from decimal import Decimal
import xlsxwriter

# Call the parameters
import ParameterSOCN
import SimulatorSOCN
#par now represents the dictionary defined in parameter function
par = ParameterSOCN.Params_6wells()




import pandas as pd
#Retrieve initial guesses for the differential states(x0), algebraic states(z0) and
#controlled variables(u0). Data listed in excel, comma separated files.
x0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/x06Sep.csv',header=None).values.reshape(-1)
z0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/z06Sep.csv',header=None).values.reshape(-1)
u0 = pd.read_csv('DatafolderSOCProdGasLimitWhN5/u06Sep.csv',header=None).values.reshape(-1)

#Retrieve the lower and upper bounds for the differential states(x), algebraic states(z) and
#controlled variables(u). Data listed in excel, comma separated files.
lbx = pd.read_csv('DatafolderSOCProdGasLimitWhN5/lbx6Sep.csv',header=None).values.reshape(-1)
lbz = pd.read_csv('DatafolderSOCProdGasLimitWhN5/lbz6Sep.csv',header=None).values.reshape(-1)
lbu = pd.read_csv('DatafolderSOCProdGasLimitWhN5/lbu6Sep.csv',header=None).values.reshape(-1)
ubx = pd.read_csv('DatafolderSOCProdGasLimitWhN5/ubx6Sep.csv',header=None).values.reshape(-1)
ubz = pd.read_csv('DatafolderSOCProdGasLimitWhN5/ubz6Sep.csv',header=None).values.reshape(-1)
ubu = pd.read_csv('DatafolderSOCProdGasLimitWhN5/ubu6Sep.csv',header=None).values.reshape(-1)

x0O = x0
z0O = z0
u0O = u0



#Define the parameter intial values(constant, if not manually changed)
p0 = ca.vertcat(par['GOR'],par['wmax_gl'],par['wmax_pg'],par['Powmax_glcom'],
                par['p_go'],par['p_oo'],par['omega1'],par['omega2'],par['omega3'])

def Wn():
    #Define error related to the measurement in %
    errorP = 0.0025
    errorF = 1
    GLC2case = []
    F,x_var, z_var, u_var, p_var, alg, dif, L, g_var =  SimulatorSOCN.CentralizedSimulator_F(par)
    nu = u_var.shape[0]
    nx = x_var.shape[0]
    
    #Define equality constraints of nlp, equals the model equations
    eqcon = ca.vertcat(alg, dif) 
    x = ca.vertcat(u_var, x_var, z_var)
    
    #Define the optimization problem(x = states, L= objective function, g = inequality constraints, p = parameters)
    nlp = {
           'x': ca.vertcat(u_var, x_var, z_var),
           'f': L,
           'g': ca.vertcat(eqcon, g_var),
           'p': p_var,
           }
    
    #Define upper/lower bounds for the inequality constraints
    lbg = ca.vertcat(np.full(eqcon.shape, 0), np.full(g_var.shape, -ca.inf))
    ubg = ca.vertcat(np.full(eqcon.shape, 0), np.full(g_var.shape, 0))
    
    #Use IPOPT(interior point optimizer) fromt the CasADI framework to solve the nlp
    opt_inst = ca.nlpsol('opt_inst', 'ipopt', nlp)
    
    #Extract the solution of the optimization, feed ipopt initial values, lower and upper bounds
    opt_res = (opt_inst(p=p0,x0 = ca.vertcat(u0,x0,z0), lbx = ca.vertcat(lbu,lbx,lbz), ubx = ca.vertcat(ubu,ubx,ubz), lbg=lbg, ubg=ubg))
    #u0,x0,z0
    states = opt_res['x']
    cost = opt_res['f']
    Wn = np.eye((15))
    
    GLC2case.append(states[59].full()) #Wellhead pressure 2
    GLC2case.append(states[53].full()) #Annulus pressure 2
    GLC2case.append(states[71].full()) #Bottomhole pressure 2
    GLC2case.append(states[63].full()) #Wellhead pressure 6
    GLC2case.append(states[57].full()) #Annulus pressure 6
    GLC2case.append(states[75].full()) #Bottomhole pressure 6
    GLC2case.append(states[126].full()) #Manifold pressure
    GLC2case.append(states[49].full()) #Discharge pressure comp 3
    GLC2case.append(states[40].full()) #Separator pressure
    GLC2case.append(states[107].full()) #Oil flow 2
    GLC2case.append(states[101].full()) #Gas flow 2
    GLC2case.append(states[111].full()) #Oil flow 6
    GLC2case.append(states[105].full()) #Gas flow 6
    GLC2case.append(states[130].full()) #Oil out separator
    GLC2case.append(states[131].full()) #Produced gas
    GLC2case.append(states[136].full()) #Tot gaslift
    #Use the nominal state to find the measurement error related to the variables
    for i in range(len(GLC2case)):
        for j in range(len(GLC2case)):
            if i == j and i <= 8: #8
                Wn[i][j] = (GLC2case[i]/100)*errorP
            if i == j and i > 7: 
                Wn[i][j] = (GLC2case[i]/100)*errorF
           
    return Wn


matWn = Wn()

workbook = xlsxwriter.Workbook('DataForBandB/Wn.xlsx')
 
#Add workbook
worksheet = workbook.add_worksheet()
 
row = 0
col = 0
 
# Iterate over the data and write it out row by row.
for d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12,d13,d14,d15 in (matWn):
    worksheet.write(row, col, d1)
    worksheet.write(row, col + 1, d2)
    worksheet.write(row, col + 2, d3)
    worksheet.write(row, col + 3, d4)
    worksheet.write(row, col + 4, d5)
    worksheet.write(row, col + 5, d6)
    worksheet.write(row, col + 6, d7)
    worksheet.write(row, col + 7, d8)
    worksheet.write(row, col + 8, d9)
    worksheet.write(row, col + 8, d9)
    worksheet.write(row, col + 9, d10)
    worksheet.write(row, col + 10, d11)
    worksheet.write(row, col + 11, d12)
    worksheet.write(row, col + 12, d13)
    worksheet.write(row, col + 13, d14)
    worksheet.write(row, col + 14, d15)
    
    row += 1
 
#Close workbook
workbook.close()
